#pragma once
#include "FBullCowGame.h"
#include <map>
#include<cctype>

//to make syntax Unreal friendly
#define TMap std::map
using FText = std::string;

using int32 = int;

FBullCowGame::FBullCowGame() // default constructor
{
	Reset();
}

int32 FBullCowGame::GetCurrentTry() const
{
	return MyCurrentTry;
}

void FBullCowGame::GetLevel(FString Level)
{
	if (Level == "1")
	{
		const FString HIDDEN_WORD = "hat"; //this must be an isogram
		MyHiddenWord = HIDDEN_WORD;
	}
	else if (Level == "2")
	{
		const FString HIDDEN_WORD = "duck";
		MyHiddenWord = HIDDEN_WORD;
	}
	else if (Level == "3")
	{
		const FString HIDDEN_WORD = "quack";
		MyHiddenWord = HIDDEN_WORD;
	}
	else if (Level == "4")
	{
		const FString HIDDEN_WORD = "python";
		MyHiddenWord = HIDDEN_WORD;
	}
	else if (Level == "5")
	{
		const FString HIDDEN_WORD = "duckies";
		MyHiddenWord = HIDDEN_WORD;
	}
}

FString FBullCowGame::GetMaxScore() 
{
	return 0;
}

int32 FBullCowGame::GetHiddenWordLength() const
{
	return MyHiddenWord.length();
}

int32 FBullCowGame::GetMaxTries() const
{
	TMap<int32, int32> WordLengthToMaxTries{ {3, 7}, {4, 6}, {5, 10}, {6, 15}, {7, 19} };
	return WordLengthToMaxTries[MyHiddenWord.length()];
}

void FBullCowGame::Reset()
{
	
	 MyCurrentTry = 1;
	 bGameIsWon = false;
	return;
}


void FBullCowGame::NewWord()
{

}


bool FBullCowGame::IsGameWon() const
{
		return bGameIsWon;
}



EGuessStatus FBullCowGame::CheckGuessValidity(FString Guess) const
{
	if (!IsIsogram(Guess)) // if the guess isn't an isogram
	{
		return EGuessStatus::Not_Isogram; 
	}
	else if (!IsLowerCase(Guess)) // if the guess isn't all lowercase
	{
		return EGuessStatus::Not_Lowercase;
	}
	else if (Guess.length() != GetHiddenWordLength()) // if the guess length is wrong
	{
		return EGuessStatus::Wrong_Length;
	}	
	else //otherwise
	{
		return EGuessStatus::OK;
	}
}

ELevelSelect FBullCowGame::CheckValidLevelInput(FString Level) const
{
	if (!NotNumber(Level))
	{
		return ELevelSelect::Input_Not_Number;
	}
	else if (Level > "5")
	{
		return ELevelSelect::Input_Too_High;
	}
	else if (Level < "1")
	{
		return ELevelSelect::Input_Too_Low;
	}
	else
	{
		return ELevelSelect::Correct;
	}
}

// recieves a VALID guess, increments turn, and returns count
FBullCowCount FBullCowGame::SubmitValidGuess(FString Guess)
{
	MyCurrentTry++;
	// setup a return varible
	FBullCowCount BullCowCount;
	int32 WordLength = MyHiddenWord.length(); // assuming the same length as Guess
	//loopthrough all letters in the guess
	bool bBulls = BullCowCount.Bulls;
	for (int32 MHWChar = 0; MHWChar < WordLength; MHWChar++) {
		//compare letters against the hidden word
		for (int32 GChar = 0; GChar < WordLength; GChar++) {
			// if they match then
			if (Guess[GChar] == MyHiddenWord[MHWChar]) {
				// increment bulls if they are in the same place
				if (MHWChar == GChar) 
					BullCowCount.Bulls++;
				
				
				// increment cows if they are not
				else 
				BullCowCount.Cows++; //must be a cow
				
			}
		}
	} if (BullCowCount.Bulls == WordLength)
	{
		bGameIsWon = true;
	}
	else 
	{
		bGameIsWon = false;
	}
	return BullCowCount;
}

bool FBullCowGame::NotNumber(FString Level) const
{
	
	for (auto character : Level)
	{
		if (isdigit(character))
		{
			return true;
		}
	}
	return false;
}

bool FBullCowGame::IsIsogram(FString Word) const
{
	//treat 0 and 1 letter words as isograms
	if (Word.length() <= 1) { return true; }

	TMap<char, bool> LetterSeen; // setup our map
	for (auto Letter : Word) // for all letters of the word;  ":" means "in"
	{// loop through all the letters of the word
		Letter = tolower(Letter); // handle mixed case, by "making upper case letters lower case in the eyes of the program" 
		if (LetterSeen[Letter])//if the letter is in the map
		{  
			return false;//we do NOT have an isogram
		}
		else // otherwise
		{  
			LetterSeen[Letter] = true;// add the letter to the map as seen
		}
		
	}

	return true; // for example in cases where \0 is entered
}

bool FBullCowGame::IsLowerCase(FString Word) const
{
	for (auto Letter : Word) 
	{
		if (!islower(Letter)) // if not a lowercase letter
		{
			return false;
		}
	}
	return true;
}
