/*This is the console executable that makes use of the bullCow class this acts as the view in a MVC pattern and is responsible for all user interaction. For game logic see the FBullCOwGame class.

*/

#pragma once

#include <iostream>
#include <string>
#include "FBullCowGame.h"

// to make syntax Ureal friendly
using FText = std::string;
using int32 = int;


// function prototypes as outside a class
void PrintIntro();
void PlayGame();
void PrintGameSummary();
void AssignLevel();
FText AskDifficulty(); // ask player for desired difficulty (will determine word length)
FText GetValidGuess();
bool AskToPlayAgain();

FBullCowGame BCGame; // instantiate a new game, which we re-use across plays




//The entry point for the application
int main()
{
	bool bPlayAgain = false;
	do {
		PrintIntro();
		//AssignLevel();
		PlayGame();
		bPlayAgain = AskToPlayAgain();
	} while (bPlayAgain);
	
	return 0;//Exit the application
}


// introduce the game to the player
void PrintIntro() 
{
	std::cout << "\n\n Welcome to Bulls and Cows, a fun word game.\n";
	std::cout << std::endl;
	std::cout << "      \\_{_(___)_}_/    ^___^ " << std::endl;
	std::cout << "          (o o)        (o o) " << std::endl;
	std::cout << "   /-------\\ /          \{ }-------\\ " << std::endl;
	std::cout << "  / | BULL |O            ^| COW  | \\ " << std::endl;
	std::cout << " *  |-,--- |              |--__--|  * " << std::endl;
	std::cout << "    ^      ^              ^      ^ " << std::endl;
	std::cout << "The object of this game is to guess the Isogram.\nAn isogram is a word that does not contain any repeating letters.\n";
	std::cout << "After you submit a guess, you will recieve a count of your 'Bulls' and 'Cows',\nthese are your clues that indicate how cloes you are to guessing the right word.\n" << "\nThe number of 'Bulls' indicates the number of letters in your guess that are correct\n\n" << "The number of 'Cows' indicates the number of letters that are the right letters but in the wrong spot.\n";
	std::cout << std::endl;
	return;
}

// plays a single game to completion
void PlayGame()
{
	BCGame.Reset();
	BCGame.GetLevel(AskDifficulty());
	int32 MaxTries = BCGame.GetMaxTries();
	
	//Loop asking for guesses while the game is Not won
	//and there are still trys remaining
	while (!BCGame.IsGameWon() && BCGame.GetCurrentTry() <= MaxTries)
	{
		FText Guess = GetValidGuess();

		
		// submit valid guess to the game
		FBullCowCount BullCowCount = BCGame.SubmitValidGuess(Guess);
		std::cout << "Bulls = " << BullCowCount.Bulls;
		std::cout << ". Cows = " << BullCowCount.Cows << "\n\n";
	}
	
	PrintGameSummary();
	return;
}
	// loop continualy until user gives a valid guess
FText GetValidGuess() 
{
	FText Guess = "";
	EGuessStatus Status = EGuessStatus::Invalid_Status;
	do {
		int32 MyCurrentTry = BCGame.GetCurrentTry();
		//get a guess from the player
		std::cout << "Can you guess the " << BCGame.GetHiddenWordLength();
		std::cout << " letter isogram I'm thinking of?\n";
		std::cout << "Try " << MyCurrentTry << " of " << BCGame.GetMaxTries() << ". Guess the word: ";
		std::getline(std::cin, Guess);
		

		Status = BCGame.CheckGuessValidity(Guess);
		switch (Status)
		{
		case EGuessStatus::Wrong_Length:
			std::cout << "Please enter a " << BCGame.GetHiddenWordLength() << " letter word.\n\n";
			break;
		case EGuessStatus::Not_Isogram:
			std::cout << "Please enter an Isogram.\n An Isogram is a word that does not contain repeating letters.";
			break;
		case EGuessStatus::Not_Lowercase:
			std::cout << "Please enter lowercase text.\n\n";
		default:
			// assuming the guess is valid
			break;
		}
		
	} while (Status != EGuessStatus::OK); // keep looping until we get no errors.
	return Guess;
}

bool AskToPlayAgain()
{
	std::cout << "Do you want to play again(y/n)?";
	FText Response = "";
	std::getline(std::cin, Response);

	return (Response[0] == 'y') || (Response[0] == 'Y');
	
	std::cout << std::endl;
}

void PrintGameSummary()
{
	if (BCGame.IsGameWon())
	{
		std::cout << "Well Done - You're a Winner!\n";
	}
	else
	{
		std::cout << "You Lose!\n Better luck next time!\n";
	}
}

void AssignLevel()
{
	FText Level = AskDifficulty();
	
	BCGame.GetLevel(Level);
}

FText AskDifficulty()
{	
	FText Level = "";
	ELevelSelect Difficulty;
	do
	{
		std::cout << "Please choose a difficulty level.\n" << "Difficulty Levels are:\n";
		std::cout << "Very Easy (Three letter word) ........... Input 1\n" << "Easy (Four letter word) ........... Input 2\n";
		std::cout << "Medium (Five leter word) ........... Input 3\n" << "Hard (Six letter word) ........... Input 4\n";
		std::cout << "Extremly Hard (Seven letter word) ........... Input 5\n";
		std::getline(std::cin, Level);

		Difficulty = BCGame.CheckValidLevelInput(Level);
		switch (Difficulty)
		{
		case ELevelSelect::Input_Too_Low:
			std::cout << "Your input of: '" << Level << "' is Too low, Please input a number from 1-5\n\n";
			break;
		case ELevelSelect::Input_Too_High:
			std::cout << "Your input of: '" << Level << "' is Too High, Please input a number from 1-5\n\n";
			break;
		case ELevelSelect::Input_Not_Number:
			std::cout << "Your input of: '" << Level << "' is not even in the realm of what would be considered valid input! Please input a NUMBER from 1-5 this time\n\n";
		}

	} while (Difficulty != ELevelSelect::Correct);
	return Level;
	
}
